/**************************************************************************//**
*   @file   AD5446.c
*   @brief  Implementation of AD5446 Driver for Microblaze processor.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define AD5446_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "ad5446.h"
#include <altera_avalon_pio_regs.h>	// PIO access macros
#include <altera_avalon_spi.h>		// SPI
#include <unistd.h>					// usleep();
#include <system.h>

#if (USE_AD5446 == 1)

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
volatile int*	AD5446_dacSlider	= &AD5446_Data[0].iData;
volatile char*	AD5446_ClkEdge  	= &AD5446_Data[1].chData;
volatile char*  AD5446_DisSDO       = &AD5446_Data[2].chData;

static char prevClkEdge = 0;
static char prevDisableSDO = 0;
static int prevDacSlider = 0;
/******************************************************************************
 * @brief Initialize SPI and Initial Values for AD5446 Board.
 *
 * @param None.
 *
 * @return TRUE or FALSE.
******************************************************************************/
int AD5446_Init( void )

{
	// setup SPI interface
	SetCtrl(AD5446_SPI_CFG);
		
	return(SUCCESS);
}

/******************************************************************************
 * @brief Update the DAC Value register in the AD5446
 *
 * @param rxData - DAC Value to be programmed
 *
 * @return None.
******************************************************************************/
void AD5446_WriteUpdateDacReg(short rxData)
{
	int cfgValue = 0;
	short loadVal = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select Command Type and DAC, add desired Data
	loadVal = rxData & 0x3FFF;
	cfgValue = CMD_LOAD_UP | loadVal;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5446_SPI, 1, 2, txBuffer, 0, NULL, 0);
}

/******************************************************************************
 * @brief Switch the valid data SCLK edge.
 *
 * @param None.
 *
 * @return None.
******************************************************************************/
void AD5446_SwitchEdge(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_RISING_EDGE;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5446_SPI, 1, 2, txBuffer, 0, NULL, 0);

	// setup SPI interface
	SetCtrl(AD5446_SPI_CFG);
}

/******************************************************************************
 * @brief Disable SDO for the AD5446.
 *
 * @param None.
 *
 * @return None.
******************************************************************************/
void AD5446_DisableSDO(void)
{
	int cfgValue = 0;
	unsigned char txBuffer[3] = {0, 0, 0};
	
	// Select command
	cfgValue = CMD_SDO_DIS;
	
	// Arrange data in 8 bit packets
	txBuffer[0] = (cfgValue & 0x00FF00) >> 8;
	txBuffer[1] = cfgValue & 0x0000FF;
	
	// Transfer data via SPI
	alt_avalon_spi_command(AD5446_SPI, 1, 2, txBuffer, 0, NULL, 0);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return TRUE
******************************************************************************/
int AD5446_UpdateParameters(void)
{

	if(*AD5446_ClkEdge != prevClkEdge)
	{
		AD5446_SwitchEdge();
		prevClkEdge = *AD5446_ClkEdge;
	}
	if(*AD5446_DisSDO != prevDisableSDO)
	{
		AD5446_DisableSDO();
		prevDisableSDO = *AD5446_DisSDO;
	}
	if(*AD5446_dacSlider != prevDacSlider)
	{
		AD5446_WriteUpdateDacReg(*AD5446_dacSlider);
		prevDacSlider = *AD5446_dacSlider;
	}
    return(SUCCESS);
}
#endif
